package com.acmebank.web;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Random;
import javax.servlet.AsyncContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;

@WebServlet(name = "AlertServlet", urlPatterns = { "/alerts" }, asyncSupported = true)
public class AlertServlet extends HttpServlet {
	private static final long serialVersionUID = 1L;

	@Override
	public void service(ServletRequest request, ServletResponse response)
			throws IOException, ServletException {
		// Starting asynchronous processing means that the underlying
		// connection won't be closed when the service method returns.
		final AsyncContext asyncContext = request.startAsync();

		// Just a welcome message...
		PrintWriter out = response.getWriter();
		out.println("Will send available alerts...");
		response.flushBuffer();

		// In a real application, the asynchronus context would be cached
		// somewhere longer lasting, perhaps an application-scoped bean
		// that monitors the system for alerts and passes them onto all
		// open connections via stored asynchronous contexts.

		// When we are ready, we will put the asynchronous context to work.
		asyncContext.start(new Runnable() {

			public void run() {
				// Simulating alerts being generated randomly on the server
				// side over a period of time...

				try {
					ServletRequest asyncRequest = asyncContext.getRequest();
					String account = asyncRequest.getParameter("account");

					ServletResponse asyncResponse = asyncContext.getResponse();
					PrintWriter asyncOut = asyncResponse.getWriter();

					Object lock = new Object();

					String[] alerts = { "Fraud alert!", "Overdraw alert!",
							"Rate increate alert!", "Fees due alert!",
							"Check bounce alert!" };
					Random random = new Random();
					int alertCount = random.nextInt(50);

					for (int i = 0; i < alertCount; i++) {
						asyncOut.println("Account " + account + ": "
								+ alerts[random.nextInt(alerts.length)]);
						asyncResponse.flushBuffer();

						synchronized (lock) {
							lock.wait(random.nextInt(3000));
						}
					}

					// When done, we can finally close the connection.
					asyncOut.println("No more alerts for now, timing out...");
					asyncResponse.flushBuffer();
					asyncContext.complete();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}
}
