package com.acmebank.web;

import java.io.Serializable;

import javax.enterprise.context.Conversation;
import javax.enterprise.context.ConversationScoped;
import javax.inject.Inject;
import javax.inject.Named;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;

import com.acmebank.domain.Account;
import com.acmebank.service.AccountService;

@Named
@ConversationScoped
public class Transfer implements Serializable {
	private static final long serialVersionUID = 1L;

	@Inject
	private AccountService accountService;

	@Inject
	private Conversation conversation;

	@NotNull(message = "Source bank is required.")
	@Size(min = 5, max = 30, message = "Source bank must be between 5 and 30 characters.")
	private String fromBank;

	@NotNull(message = "Source account is required.")
	@Size(min = 7, max = 30, message = "Source account must be between 7 and 30 characters.")
	private String fromAccount;

	@NotNull(message = "Destination bank is required.")
	@Size(min = 5, max = 30, message = "Destination bank must be between 5 and 30 characters.")
	private String toBank;

	@NotNull(message = "Destination account is required.")
	@Size(min = 7, max = 30, message = "Destination account must be between 7 and 30 characters.")
	private String toAccount;

	@Min(value = 0, message = "Transfer amount must be positive.")
	private double amount;

	@Inject
	public void setAccount(@SelectedAccount Account account) {
		fromBank = "Acme Bank";
		fromAccount = account.getNumber();
	}

	public String getFromAccount() {
		return fromAccount;
	}

	public void setFromAccount(String fromAccount) {
		this.fromAccount = fromAccount;
	}

	public String getFromBank() {
		return fromBank;
	}

	public void setFromBank(String fromBank) {
		this.fromBank = fromBank;
	}

	public String getToAccount() {
		return toAccount;
	}

	public void setToAccount(String toAccount) {
		this.toAccount = toAccount;
	}

	public String getToBank() {
		return toBank;
	}

	public void setToBank(String toBank) {
		this.toBank = toBank;
	}

	public double getAmount() {
		return amount;
	}

	public void setAmount(double amount) {
		this.amount = amount;
	}

	public String enterToBank() {
		conversation.begin();

		return "transfer_step_2.xhtml";
	}

	public String enterAmount() {
		return "transfer_step_3.xhtml";
	}

	public String transfer() {
		accountService.transfer(toBank, toAccount, fromBank, fromAccount,
				amount);

		conversation.end();

		return "/account.xhtml";
	}
}
